@@ -22,9 +22,12 @@
 complete_names <- function(x, template, allow.invalid = FALSE)
 {
     pid <- pmatch(names(x), names(template), duplicates.ok = TRUE)
-    if (any(is.na(pid))) {
-        if (allow.invalid) pid <- pid[!is.na(pid)]
-        else warning("Invalid or ambiguous component names: ",
+    if (allow.invalid) {
+        x <- x[!is.na(pid)]
+        pid <- pid[!is.na(pid)]
+    } else {
+        if (any(is.na(pid)))
+            warning("Invalid or ambiguous component names: ",
                      paste(names(x)[which(is.na(pid))], collapse = ", ") )
     }
     if (any(duplicated(pid))) stop("Multiple matches to component name")
